@@ -16,8 +16,8 @@ from lxml import etree
 #### Pages/lines
 
 # How many pages/lines to scan when finding header/footer automatically
-PAGE_SCAN_COUNT = 20		# Arbitrary
-LINE_SCAN_COUNT = 2		# Arbitrary
+PAGE_SCAN_COUNT = 20    # Arbitrary
+LINE_SCAN_COUNT = 2     # Arbitrary
 
 # Number of character widths that two strings have to be apart,
 # for them to be considered part of the same text fragment
@@ -211,7 +211,7 @@ class Text(Element):
           and abs(other.left - self.right) < 2.0:
           #and abs(other.left - self.right) < self.average_character_width / 3.0:
             has_gap = 0
-        else:	# Insert n spaces to fill gap.  Use TAB?  Columns?
+        else:  # Insert n spaces to fill gap.  Use TAB?  Columns?
             if other.left < self.right:
                 has_gap = 1  # Coalescing different lines. 1 space
             else:    # Multiple texts on same line
@@ -275,7 +275,7 @@ class Text(Element):
         self.font_size_em = max(self.font_size_em, other.font_size_em)
         self.font = other.font if self.font_size == other.font_size else other.font
         if has_gap > 0:
-            if has_gap < 3:	# Small number of spaces = 1 space
+            if has_gap < 3:  # Small number of spaces = 1 space
                 if not (self.text_as_string.endswith(' ') \
                      or self.text_as_string.endswith('-') \
                      or other.text_as_string.startswith(' ') \
@@ -766,11 +766,11 @@ class Page:
                 while s < len(text.text_as_string) \
                   and text.text_as_string[s] == ' ':
                     s += 1
-                if s > 2:	# Allow two leading spaces
+                if s > 2:  # Allow two leading spaces
                     # Assume this is a standard indent
                     # Normally text.indented gets set later
                     text.indented = 1
-                    w = round(s * text.average_character_width/2.0)	# Spaces < avg width
+                    w = round(s * text.average_character_width/2.0)  # Spaces < avg width
                     matchObj = re.match(r'^\s*(<[^>]+>)?\s*(.*)$', text.raw)
                     t1 = matchObj.group(1)
                     t2 = matchObj.group(2)
@@ -780,9 +780,9 @@ class Page:
                         t2 = ''
                     text.raw = t1 + t2
                     text.text_as_string = text.text_as_string[s:]
-                    text.left += w	# Add indent
+                    text.left += w  # Add indent
                     text.last_left += w
-                    text.width -= w	# Reduce width
+                    text.width -= w  # Reduce width
                     text.final_width -= w
                 self.left_margin = min(text.left, self.left_margin)
                 self.right_margin = max(text.right, self.right_margin)
@@ -830,17 +830,23 @@ class Page:
           and len(self.imgs) == 0
 
     def find_match(self, frag):
+        # We have not yet worked out the stats, specifically line_spacing
+        # Approximate the line spacing for checking overlapped lines
+        line_height = frag.bottom - frag.top
         for t in self.texts:
             if t is not frag :
                 # Do the parts of a line overlap?
                 # Some files can have separate lines overlapping slightly
                 # BOTTOM_FACTOR allows for this
-                if (frag.top == t.top or frag.bottom == t.bottom) \
-                  or (frag.top < t.top and frag.bottom > t.top+BOTTOM_FACTOR) \
-                  or (frag.top < t.top and frag.bottom+BOTTOM_FACTOR > t.bottom) \
-                  or (t.top < frag.top and t.bottom > frag.top+BOTTOM_FACTOR) \
-                  or (t.top < frag.top and t.bottom+BOTTOM_FACTOR > frag.bottom):
-                    return t	# Force match if same line
+                top = min(frag.top, t.top)
+                bot = max(frag.bottom, t.bottom)
+                if bot - top < line_height * 1.5 \
+                  and ((frag.top == t.top or frag.bottom == t.bottom) \
+                    or (frag.top < t.top and frag.bottom > t.top+BOTTOM_FACTOR) \
+                    or (frag.top < t.top and frag.bottom+BOTTOM_FACTOR > t.bottom) \
+                    or (t.top < frag.top and t.bottom > frag.top+BOTTOM_FACTOR) \
+                    or (t.top < frag.top and t.bottom+BOTTOM_FACTOR > frag.bottom)):
+                    return t  # Force match if same line
                     # Sorting can put parts of a line in the wrong order if there are small chars
                     if t.left < frag.left:
                         hdelta = frag.left - t.right
@@ -910,7 +916,7 @@ class Page:
             # NB Doesn't work where Contents goes to another page
             if re.match(r'(?i)^\s*(table of )?contents\s*$', t.text_as_string) is not None:
                 self.contents = True
-                t.tag = 'h2'	# It won't get set later
+                t.tag = 'h2'  # It won't get set later
             # Centered if left and right margins are within FACTOR%
             # Because indents can waver a bit, use between indent and indent1 as == indent
             if (lmargin < indent or lmargin > indent1) \
@@ -1045,8 +1051,8 @@ class Page:
                                 # Should check for values approx the same, as with indents
                                 frag.margin_left = int(round(((frag.left - left) / self.stats_margin_px)+0.5))
                         if last_frag is not None \
-                          and frag.bottom - last_frag.bottom \
-                              > stats.para_space*SECTION_FACTOR:
+                          and stats.para_space > 0 \
+                          and frag.bottom - last_frag.bottom > stats.para_space*SECTION_FACTOR:
                           #and frag.top - last_frag.bottom > frag.height + stats.line_space + (stats.line_space*LINE_FACTOR):
                             frag.blank_line_before = 1
                 last_frag = frag
@@ -1103,7 +1109,7 @@ class Page:
         # Do this before automatic actions
         self.remove_head_foot_regex(opts)
 
-    def find_margins(self, tops, indents_odd, indents_even, line_spaces, bottoms, rights):
+    def find_margins(self, tops, indents, line_spaces, bottoms, rights):
 
         #from collections import Counter
 
@@ -1112,6 +1118,7 @@ class Page:
         # The most used font will be treated as size 1em
         max_bot = 0
         max_right = 0
+        max_space = 0
         last_top = 0
         #last_bottom = 0
         first = True
@@ -1132,21 +1139,23 @@ class Page:
                 # Beware of multiple text on same line. These look like small spacing
                 if text.height <= space:
                     line_spaces[space] = line_spaces.get(space, 0) + 1
+                elif not max_space:
+                    max_space = space  # Remember first in case of short docs
 
             last_top = top
             max_bot = max(max_bot, text.bottom)
 
             max_right = max(max_right, text.right)
 
-            if self.odd_even:
-                indents_odd[left] = indents_odd.get(left, 0) + 1
-            else:
-                indents_even[left] = indents_even.get(left, 0) + 1
+            indents[left] = indents.get(left, 0) + 1
 
         if max_bot > 0:
             bottoms[max_bot] = bottoms.get(max_bot, 0) + 1
         if max_right > 0:
             rights[max_right] = rights.get(max_right, 0) + 1
+        if max_space > 0 and not line_spaces:
+            # Nothing has been set up, so create one to avoid empty array
+            line_spaces[max_space] = line_spaces.get(max_space, 0) + 1
 
         return
         #########################
@@ -1398,9 +1407,9 @@ class PDFDocument:
 
         # Check for a testable value
         if self.opts.pdf_header_regex is None:
-            self.opts.pdf_header_regex = ''	# Do nothing
+            self.opts.pdf_header_regex = ''  # Do nothing
         if self.opts.pdf_footer_regex is None:
-            self.opts.pdf_footer_regex = ''	# Do nothing
+            self.opts.pdf_footer_regex = ''  # Do nothing
 
         parser = etree.XMLParser(recover=True)
         self.root = etree.fromstring(xml, parser=parser)
@@ -1449,8 +1458,8 @@ class PDFDocument:
 
         # Work out document dimensions from page format
         for page in self.pages:
-            page.find_margins(self.tops, self.indents_odd, self.indents_even, \
-                             self.line_spaces, self.bottoms, self.rights)
+            page.find_margins(self.tops, self.indents_odd if page.odd_even else self.indents_even, \
+                              self.line_spaces, self.bottoms, self.rights)
 
         self.setup_stats()
 
@@ -1720,7 +1729,9 @@ class PDFDocument:
                 break
             count -= 1
 
-        # For safety, check in the right order
+        # For safety, check present and in the right order
+        if not para_k or para_k == line_k:
+            para_k = round(line_k * PARA_FACTOR)
         if line_k > para_k:
             x = para_k
             para_k = line_k
@@ -1729,7 +1740,7 @@ class PDFDocument:
         self.stats.line_space = line_k
         # Some docs have no great distinction for paragraphs
         # Limit the size of the gap, or section breaks not found
-        if para_k > line_k * PARA_FACTOR:
+        if para_k > round(line_k * PARA_FACTOR):
             self.stats.para_space = round(line_k * PARA_FACTOR)
         else:
             self.stats.para_space = para_k
@@ -1757,14 +1768,21 @@ class PDFDocument:
         head_text = [''] * LINE_SCAN_COUNT
         head_match = [0] * LINE_SCAN_COUNT
         head_match1 = [0] * LINE_SCAN_COUNT
+        head_match2 = [0] * LINE_SCAN_COUNT
         head_page = 0
         head_skip = 0
         foot_text = [''] * LINE_SCAN_COUNT
         foot_match = [0] * LINE_SCAN_COUNT
         foot_match1 = [0] * LINE_SCAN_COUNT
+        foot_match2 = [0] * LINE_SCAN_COUNT
         foot_page = 0
         foot_skip = 0
+        # xxx nn xxx nn  or nn xxx  or just roman numerals
         pagenum_text = r'(.*\d+\s+\w+\s+\d+.*)|(\s*\d+\s+.*)|(^\s*[ivxlcIVXLC]+\s*$)'
+        # For line ending nn, is the preceding text constant
+        fixed_text = r'(^.+[^0-9])\d+\s*$'
+        fixed_head = ''
+        fixed_foot = ''
 
         pages_to_scan = scan_count
         # Note that a line may be in more than 1 part
@@ -1790,11 +1808,21 @@ class PDFDocument:
                             head_match[head_ind] += 1
                             if head_page == 0:
                                 head_page = page.number
-                        else:	# Look for page count of format 'n xxx n'
-                            if re.match(pagenum_text, t) is not None:
-                                head_match1[head_ind] += 1
-                                if head_page == 0:
-                                    head_page = page.number
+                        elif re.match(pagenum_text, t) is not None:
+                            # Look for page count of format 'n xxx n'
+                            head_match1[head_ind] += 1
+                            if head_page == 0:
+                                head_page = page.number
+                        else:
+                            # Look for text of format 'constant nn'
+                            f = re.match(fixed_text, t)
+                            if f and f.group(1):
+                                if not fixed_head:
+                                    fixed_head = f.group(1)
+                                elif fixed_head == f.group(1):
+                                    head_match2[head_ind] += 1
+                                    if head_page == 0:
+                                        head_page = page.number
 
             if self.opts.pdf_footer_skip < 0 \
               and len(page.texts) > 0:
@@ -1813,11 +1841,21 @@ class PDFDocument:
                             foot_match[foot_ind] += 1
                             if foot_page == 0:
                                 foot_page = page.number
-                        else:	# Look for page count of format 'n xxx n'
-                            if re.match(pagenum_text, t) is not None:
-                                foot_match1[foot_ind] += 1
-                                if foot_page == 0:
-                                    foot_page = page.number
+                        elif re.match(pagenum_text, t) is not None:
+                            # Look for page count of format 'n xxx n'
+                            foot_match1[foot_ind] += 1
+                            if foot_page == 0:
+                                foot_page = page.number
+                        else:
+                            # Look for text of format 'constant nn'
+                            f = re.match(fixed_text, t)
+                            if f and f.group(1):
+                                if not fixed_foot:
+                                    fixed_foot = f.group(1)
+                                elif fixed_foot == f.group(1):
+                                    foot_match2[foot_ind] += 1
+                                    if foot_page == 0:
+                                        foot_page = page.number
 
             pages_to_scan -= 1
             if pages_to_scan < 1:
@@ -1825,27 +1863,35 @@ class PDFDocument:
 
         if pages_to_scan > 0:
             # Doc is shorter than scan_count
-            pages_to_scan = scan_count - pages_to_scan	# Number scanned
+            pages_to_scan = scan_count - pages_to_scan  # Number scanned
         else:
             # All required pages scanned
             pages_to_scan = scan_count
-        pages_to_scan /= 2	# Are at least half matching?
+        pages_to_scan /= 2  # Are at least half matching?
 
         head_ind = 0
         for i in range(LINE_SCAN_COUNT):
-            if head_match[i] > pages_to_scan or head_match1[i] > pages_to_scan:
+            if head_match[i] > pages_to_scan \
+              or head_match1[i] > pages_to_scan \
+              or head_match2[i] > pages_to_scan:
                 head_ind = i  # Remember the last matching line
         if self.pages[head_page].texts \
-          and (head_match[head_ind] > pages_to_scan or head_match1[head_ind] > pages_to_scan):
+          and (head_match[head_ind] > pages_to_scan \
+            or head_match1[head_ind] > pages_to_scan \
+            or head_match2[head_ind] > pages_to_scan):
             t = self.pages[head_page].texts[head_ind]
             head_skip = t.top + t.height + 1
 
         foot_ind = 0
         for i in range(LINE_SCAN_COUNT):
-            if foot_match[i] > pages_to_scan or foot_match1[i] > pages_to_scan:
+            if foot_match[i] > pages_to_scan \
+              or foot_match1[i] > pages_to_scan \
+              or foot_match2[i] > pages_to_scan:
                 foot_ind = i  # Remember the last matching line
         if self.pages[foot_page].texts \
-          and (foot_match[foot_ind] > pages_to_scan or foot_match1[foot_ind] > pages_to_scan):
+          and (foot_match[foot_ind] > pages_to_scan \
+            or foot_match1[foot_ind] > pages_to_scan \
+            or foot_match2[foot_ind] > pages_to_scan):
             t = self.pages[foot_page].texts[-foot_ind-1]
             foot_skip = t.top - 1
 
@@ -1884,6 +1930,8 @@ class PDFDocument:
         save_bottom = 0
         # After merge, skip to this page
         pind = 0
+        # If a page is merged, and removed, may need to remember it
+        save_candidate = None
 
         # Now merge where bottom of one is within ORPHAN_LINES lines of max_bottom
         # and top of next is within a line of min_top
@@ -1892,7 +1940,8 @@ class PDFDocument:
         while merge_done:
             merge_done = False  # A merge was done
             merged_page = None  # Page merged into previous
-            candidate = None    # Lines close enough to the bottom that it might merge
+            candidate = save_candidate  # Lines close enough to the bottom that it might merge
+            save_candidate = None
             while pind < len(self.pages):
                 page = self.pages[pind]
                 stats_left = page.stats_left
@@ -1976,14 +2025,19 @@ class PDFDocument:
                 candidate.texts[-1].coalesce(merged_text, candidate.number, left_margin, right_margin)
                 merged_page.texts.remove(merged_text)
                 # Put back top/bottom after coalesce if final line
-                if save_bottom != 0.0 :
+                if save_bottom:
                     # Ignore top as that can confuse things where the 1st para of a page
                     # was merged with a previous.  Keep the original top
                     candidate.texts[-1].bottom = save_bottom
-                #candidate.coalesce_paras()
+
                 # Have we removed everything from this page (well, all texts and images)
                 if merged_page.is_empty:
-                    candidate.texts[-1].blank_line_before = 1
+                    # Empty page does/may not actually mean blank line
+                    #candidate.texts[-1].blank_line_before = 1
+                    # If pages are merged, and the merged page gets removed (as here),
+                    # and the next page is short (forced page break),
+                    # then the merge would fail when this loop restarts.
+                    save_candidate = candidate
                     self.pages.remove(merged_page)
 
     def linearize(self):
